import * as React from 'react';
import { render } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import Modal from './Modal';

const mockOkayClick = jest.fn();
const mockDefaultAction = jest.fn();
const closeCallback = jest.fn();

jest.unmock('./Modal');

describe('<Modal ..>', () => {
  it.each([
    ['esc', closeCallback],
    ['enter', mockDefaultAction],
  ])('the Modal can call callbacks for key %s', (key, callback) => {
    render(
      <Modal
        header="Coffee"
        defaultActionCallback={mockDefaultAction}
        closeCallback={closeCallback}
        buttons={<></>}
      >
        <strong>double shot</strong>
      </Modal>,
    );
    const body = document.getElementsByTagName('body')[0];
    userEvent.type(body, `{${key}}`);
    expect(callback).toBeCalled();
  });

  test(`The modal renders buttons`, () => {
    const { getByRole } = render(
      <Modal
        header="Coffee"
        buttons={<button onClick={mockOkayClick}>Make coffee</button>}
      >
        <strong>double shot</strong>
      </Modal>,
    );
    expect(getByRole('button', { name: /make coffee/i })).toBeInTheDocument();
  });

  test(`modal default options`, () => {
    const { getByRole } = render(
      <Modal
        header="Coffee"
        buttons={<></>}
        curtain={true}
        closeButton={true}
        closeCallback={closeCallback}
      >
        <strong>double shot</strong>
      </Modal>,
    );
    const wrapper = document.getElementsByClassName('modal-wrapper')[0];
    expect(wrapper).toHaveClass('curtain');
    userEvent.click(getByRole('button', { name: 'close' }));
    expect(closeCallback).toBeCalled();
  });

  test(`modal non-default options`, () => {
    const { queryByRole } = render(
      <Modal
        header="Coffee"
        buttons={<></>}
        curtain={false}
        closeButton={false}
        closeCallback={closeCallback}
      >
        <strong>double shot</strong>
      </Modal>,
    );
    const wrapper = document.getElementsByClassName('modal-wrapper')[0];
    expect(wrapper).not.toHaveClass('curtain');
    expect(queryByRole('button', { name: 'close' })).not.toBeInTheDocument();
  });

  it(`Get an error when passed a bad dom node selector`, () => {
    const { getByText } = render(
      <Modal header="Coffee" DOMNodeSelector=".badselector" buttons={<></>}>
        <strong>double shot</strong>
      </Modal>,
    );
    expect(getByText(/did not yield a dom node/i)).toBeInTheDocument();
  });
});
