import React, { useEffect } from 'react';
import ReactDOM from 'react-dom';

export interface ModalProps {
  DOMNodeSelector?: string;
  curtain?: boolean;
  children: React.ReactNode;
  header: React.ReactNode;
  defaultActionCallback?: CallableFunction | undefined;
  closeCallback?: CallableFunction | undefined;
  closeButton?: boolean;
  buttons: React.ReactNode;
}

export const Modal: React.FC<ModalProps> = ({
  children,
  closeCallback,
  defaultActionCallback,
  header,
  buttons,
  DOMNodeSelector = 'body',
  curtain = true,
  closeButton = true,
}: ModalProps) => {
  const wrapper = document.createElement('div');
  wrapper.classList.add('modal-wrapper');
  if (curtain) wrapper.classList.add('curtain');
  const portal = document.querySelector(DOMNodeSelector);
  if (!portal)
    return (
      <strong>
        Querying &quot;{DOMNodeSelector}&quot; did not yield a DOM node
      </strong>
    );

  // Mount and unmount the modal in the portal
  useEffect(() => {
    portal.appendChild(wrapper);
    portal.classList.add('modal-active');
    return () => {
      portal.removeChild(wrapper);
      portal.classList.remove('modal-active');
    };
  }, []);

  const keyUpHandler = (event: KeyboardEvent) => {
    if (event.key == 'Escape' && typeof closeCallback == 'function')
      closeCallback();
    if (event.key == 'Enter' && typeof defaultActionCallback == 'function')
      defaultActionCallback();
  };

  useEffect(() => {
    window.addEventListener('keyup', keyUpHandler);
    return () => {
      window.removeEventListener('keyup', keyUpHandler);
    };
  });

  const randomEnough = (+new Date()).toString(36).slice(-10);

  const close = () => typeof closeCallback == 'function' && closeCallback();
  return ReactDOM.createPortal(
    <div
      className="modal"
      role="dialog"
      aria-labelledby={`${randomEnough}-header`}
      aria-describedby={`${randomEnough}-main`}
    >
      <header id={`${randomEnough}-header`}>
        <h5>{header}</h5>
        {closeButton && (
          <button className="close" onClick={close} aria-label="close">
            ⨯
          </button>
        )}
      </header>
      <main id={`${randomEnough}-main`}>{children}</main>
      <footer>{buttons}</footer>
    </div>,
    wrapper,
  );
};

export default Modal;
